from abc import abstractmethod
from typing import Protocol

from clifinance.domain.expense.model import (
    Expense,
    ExpenseAmount,
    ExpenseDate,
    ExpenseType,
)


class ExpenseGateway(Protocol):
    @abstractmethod
    def get_all_expenses(self) -> list[Expense]:
        raise NotImplementedError

    @abstractmethod
    def add_expense(self, exp: Expense) -> None:
        raise NotImplementedError

    @abstractmethod
    def update_expense(self, expense: Expense) -> None:
        raise NotImplementedError

    @abstractmethod
    def get_all_expenses_by_type(self, type: ExpenseType) -> list[Expense]:
        raise NotImplementedError

    @abstractmethod
    def get_all_expenses_by_date(self, date: ExpenseDate) -> list[Expense]:
        raise NotImplementedError

    @abstractmethod
    def get_all_expenses_by_amount(self, amount: ExpenseAmount) -> list[Expense]:
        raise NotImplementedError

    @abstractmethod
    def get_all_expenses_by_filter(
        self,
        type: ExpenseType | None,
        date: ExpenseDate | None,
        amount: ExpenseAmount | None,
    ) -> list[Expense]:
        raise NotImplementedError
