from clifinance.application.abstractions.protocols.session import Session
from clifinance.domain.expense.gateway import ExpenseGateway
from clifinance.domain.expense.model import (
    Expense,
    ExpenseAmount,
    ExpenseDate,
    ExpenseType,
)


class JsonExpenseGateway(ExpenseGateway):
    def __init__(self, session: Session) -> None:
        self.session = session

    def get_all_expenses(self) -> list[Expense]:
        return self.session.get_all()

    def add_expense(self, exp: Expense) -> None:
        self.session.add(exp)
        self.session.commit()

    def update_expense(self, expense: Expense) -> None:
        self.session.update(expense)

    def get_all_expenses_by_type(self, type: ExpenseType) -> list[Expense]:
        return [x for x in self.session.get_all() if x.type == type]

    def get_all_expenses_by_date(self, date: ExpenseDate) -> list[Expense]:
        return [x for x in self.session.get_all() if x.date == date]

    def get_all_expenses_by_amount(self, amount: ExpenseAmount) -> list[Expense]:
        return [x for x in self.session.get_all() if x.amount == amount]

    def get_all_expenses_by_filter(
        self,
        type: ExpenseType | None,
        date: ExpenseDate | None,
        amount: ExpenseAmount | None,
    ) -> list[Expense]:

        result = self.get_all_expenses()
        if type is not None:
            result = [x for x in result if x.type == type]
        if date is not None:
            result = [x for x in result if x.date == date]
        if amount is not None:
            result = [x for x in result if x.amount.value == amount.value]
        return result
